@@ -17,14 +17,15 @@ void generate_periodic_mesh(
     const bool odt = false,
     const bool perturb = true,
     const bool exude = true,
-    const double edge_size = 0.0,  // std::numeric_limits<double>::max(),
-    const double facet_angle = 0.0,
-    const double facet_size = 0.0,
-    const double facet_distance = 0.0,
-    const double cell_radius_edge_ratio = 0.0,
-    const double cell_size = 0.0,
+    const double max_edge_size_at_feature_edges = 0.0,  // std::numeric_limits<double>::max(),
+    const double min_facet_angle = 0.0,
+    const double max_radius_surface_delaunay_ball = 0.0,
+    const double max_facet_distance = 0.0,
+    const double max_circumradius_edge_ratio = 0.0,
+    const double max_cell_circumradius = 0.0,
     const int number_of_copies_in_output = 1,
-    const bool verbose = true
+    const bool verbose = true,
+    const int seed = 0
     );
 
 } // namespace pygalmesh
