@@ -1,5 +1,6 @@
 #include "domain.hpp"
 #include "generate.hpp"
+#include "generate_2d.hpp"
 #include "generate_from_off.hpp"
 #include "generate_from_inr.hpp"
 #include "remesh_surface.hpp"
@@ -8,7 +9,8 @@
 #include "polygon2d.hpp"
 #include "primitives.hpp"
 #include "sizing_field.hpp"
-#include "version.hpp"
+
+#include <CGAL/version.h>
 
 #include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
@@ -240,7 +242,7 @@ PYBIND11_MODULE(_pygalmesh, m) {
               py::arg("poly"),
               py::arg("direction"),
               py::arg("alpha") = 0.0,
-              py::arg("edge_size") = 0.0
+              py::arg("max_edge_size_at_feature_edges") = 0.0
               )
           .def("eval", &Extrude::eval)
           .def("get_bounding_sphere_squared_radius", &Extrude::get_bounding_sphere_squared_radius)
@@ -256,41 +258,36 @@ PYBIND11_MODULE(_pygalmesh, m) {
           .def("get_features", &ring_extrude::get_features);
 
     // functions
+    m.def(
+        "_generate_2d", &generate_2d,
+        py::arg("points"),
+        py::arg("constraints"),
+        py::arg("max_circumradius_shortest_edge_ratio") = 1.41421356237,
+        py::arg("max_edge_size") = 0.0,
+        py::arg("num_lloyd_steps") = 0
+        );
     m.def(
         "_generate_mesh", &generate_mesh,
         py::arg("domain"),
         py::arg("outfile"),
-        py::arg("feature_edges") = std::vector<std::vector<std::array<double, 3>>>(),
+        py::arg("extra_feature_edges") = std::vector<std::vector<std::array<double, 3>>>(),
         py::arg("bounding_sphere_radius") = 0.0,
         py::arg("lloyd") = false,
         py::arg("odt") = false,
         py::arg("perturb") = true,
         py::arg("exude") = true,
-        py::arg("edge_size") = 0.0,
-        py::arg("facet_angle") = 0.0,
-        py::arg("facet_size") = 0.0,
-        py::arg("facet_distance") = 0.0,
-        py::arg("cell_radius_edge_ratio") = 0.0,
-        py::arg("cell_size") = 0.0,
-        py::arg("verbose") = true
-        );
-    m.def(
-        "_generate_with_sizing_field", &generate_with_sizing_field,
-        py::arg("domain"),
-        py::arg("outfile"),
-        py::arg("feature_edges") = std::vector<std::vector<std::array<double, 3>>>(),
-        py::arg("bounding_sphere_radius") = 0.0,
-        py::arg("lloyd") = false,
-        py::arg("odt") = false,
-        py::arg("perturb") = true,
-        py::arg("exude") = true,
-        py::arg("edge_size") = 0.0,
-        py::arg("facet_angle") = 0.0,
-        py::arg("facet_size") = 0.0,
-        py::arg("facet_distance") = 0.0,
-        py::arg("cell_radius_edge_ratio") = 0.0,
-        py::arg("cell_size") = nullptr,
-        py::arg("verbose") = true
+        py::arg("max_edge_size_at_feature_edges_value") = 0.0,
+        py::arg("max_edge_size_at_feature_edges_field") = nullptr,
+        py::arg("min_facet_angle") = 0.0,
+        py::arg("max_radius_surface_delaunay_ball_value") = 0.0,
+        py::arg("max_radius_surface_delaunay_ball_field") = nullptr,
+        py::arg("max_facet_distance_value") = 0.0,
+        py::arg("max_facet_distance_field") = nullptr,
+        py::arg("max_circumradius_edge_ratio") = 0.0,
+        py::arg("max_cell_circumradius_value") = 0.0,
+        py::arg("max_cell_circumradius_field") = nullptr,
+        py::arg("verbose") = true,
+        py::arg("seed") = 0
         );
     m.def(
         "_generate_periodic_mesh", &generate_periodic_mesh,
@@ -301,24 +298,26 @@ PYBIND11_MODULE(_pygalmesh, m) {
         py::arg("odt") = false,
         py::arg("perturb") = true,
         py::arg("exude") = true,
-        py::arg("edge_size") = 0.0,
-        py::arg("facet_angle") = 0.0,
-        py::arg("facet_size") = 0.0,
-        py::arg("facet_distance") = 0.0,
-        py::arg("cell_radius_edge_ratio") = 0.0,
-        py::arg("cell_size") = 0.0,
+        py::arg("max_edge_size_at_feature_edges") = 0.0,
+        py::arg("min_facet_angle") = 0.0,
+        py::arg("max_radius_surface_delaunay_ball") = 0.0,
+        py::arg("max_facet_distance") = 0.0,
+        py::arg("max_circumradius_edge_ratio") = 0.0,
+        py::arg("max_cell_circumradius") = 0.0,
         py::arg("number_of_copies_in_output") = 1,
-        py::arg("verbose") = true
+        py::arg("verbose") = true,
+        py::arg("seed") = 0
         );
     m.def(
         "_generate_surface_mesh", &generate_surface_mesh,
         py::arg("domain"),
         py::arg("outfile"),
         py::arg("bounding_sphere_radius") = 0.0,
-        py::arg("angle_bound") = 0.0,
-        py::arg("radius_bound") = 0.0,
-        py::arg("distance_bound") = 0.0,
-        py::arg("verbose") = true
+        py::arg("min_facet_angle") = 0.0,
+        py::arg("max_radius_surface_delaunay_ball") = 0.0,
+        py::arg("max_facet_distance") = 0.0,
+        py::arg("verbose") = true,
+        py::arg("seed") = 0
         );
     m.def(
         "_generate_from_off", &generate_from_off,
@@ -328,13 +327,15 @@ PYBIND11_MODULE(_pygalmesh, m) {
         py::arg("odt") = false,
         py::arg("perturb") = true,
         py::arg("exude") = true,
-        py::arg("edge_size") = 0.0,  // std::numeric_limits<double>::max(),
-        py::arg("facet_angle") = 0.0,
-        py::arg("facet_size") = 0.0,
-        py::arg("facet_distance") = 0.0,
-        py::arg("cell_radius_edge_ratio") = 0.0,
-        py::arg("cell_size") = 0.0,
-        py::arg("verbose") = true
+        py::arg("max_edge_size_at_feature_edges") = 0.0,  // std::numeric_limits<double>::max(),
+        py::arg("min_facet_angle") = 0.0,
+        py::arg("max_radius_surface_delaunay_ball") = 0.0,
+        py::arg("max_facet_distance") = 0.0,
+        py::arg("max_circumradius_edge_ratio") = 0.0,
+        py::arg("max_cell_circumradius") = 0.0,
+        py::arg("verbose") = true,
+        py::arg("reorient") = false,
+        py::arg("seed") = 0
         );
     m.def(
         "_generate_from_inr", &generate_from_inr,
@@ -344,25 +345,44 @@ PYBIND11_MODULE(_pygalmesh, m) {
         py::arg("odt") = false,
         py::arg("perturb") = true,
         py::arg("exude") = true,
-        py::arg("edge_size") = 0.0,
-        py::arg("facet_angle") = 0.0,
-        py::arg("facet_size") = 0.0,
-        py::arg("facet_distance") = 0.0,
-        py::arg("cell_radius_edge_ratio") = 0.0,
-        py::arg("cell_size") = 0.0,
-        py::arg("verbose") = true
+        py::arg("max_edge_size_at_feature_edges") = 0.0,
+        py::arg("min_facet_angle") = 0.0,
+        py::arg("max_radius_surface_delaunay_ball") = 0.0,
+        py::arg("max_facet_distance") = 0.0,
+        py::arg("max_circumradius_edge_ratio") = 0.0,
+        py::arg("max_cell_circumradius") = 0.0,
+        py::arg("verbose") = true,
+        py::arg("seed") = 0
+        );
+    m.def(
+        "_generate_from_inr_with_subdomain_sizing", &generate_from_inr_with_subdomain_sizing,
+        py::arg("inr_filename"),
+        py::arg("outfile"),
+        py::arg("default_max_cell_circumradius"),
+        py::arg("max_cell_circumradiuss"),
+        py::arg("cell_labels"),
+        py::arg("lloyd") = false,
+        py::arg("odt") = false,
+        py::arg("perturb") = true,
+        py::arg("exude") = true,
+        py::arg("max_edge_size_at_feature_edges") = 0.0,
+        py::arg("min_facet_angle") = 0.0,
+        py::arg("max_radius_surface_delaunay_ball") = 0.0,
+        py::arg("max_facet_distance") = 0.0,
+        py::arg("max_circumradius_edge_ratio") = 0.0,
+        py::arg("verbose") = true,
+        py::arg("seed") = 0
         );
     m.def(
         "_remesh_surface", &remesh_surface,
         py::arg("infile"),
         py::arg("outfile"),
-        py::arg("edge_size") = 0.0,
-        py::arg("facet_angle") = 0.0,
-        py::arg("facet_size") = 0.0,
-        py::arg("facet_distance") = 0.0,
-        py::arg("verbose") = true
-        );
-    m.def(
-        "_get_cgal_version", &get_cgal_version
+        py::arg("max_edge_size_at_feature_edges") = 0.0,
+        py::arg("min_facet_angle") = 0.0,
+        py::arg("max_radius_surface_delaunay_ball") = 0.0,
+        py::arg("max_facet_distance") = 0.0,
+        py::arg("verbose") = true,
+        py::arg("seed") = 0
         );
+    m.attr("_CGAL_VERSION_STR") = CGAL_VERSION_STR;
 }
